#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# LpBugReporter - Is a python library for Launchdpad Bug Reporting.
#
# Copyright (C) 2010 zedtux <zedtux@zedroot.org>
#
# LpBugReporter is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# LpBugReporter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LpBugReporter; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import os
import gtk
import gobject
import logging
import StringIO
import traceback
import webbrowser

from bugreporter.logger import enable_debugging
from bugreporter.lpbugreporterthread import LpBrThread

log = logging.getLogger("LpBugReporter")

PROJECT_PATH = os.path.dirname(os.path.realpath(__file__)) + os.path.sep
VERSION = "1.0.2"
LOGO_PATH = os.path.join(PROJECT_PATH, "pixmaps", "lpbugreporter.png")
PROJECT_HOMEPAGE_LINK = "https://launchpad.net/%s"
PROJECT_BUGS_LINK = "https://launchpad.net/%s/+filebug"
PROJECT_BUG_LINK = "https://bugs.launchpad.net/%s/+bug/%s"

gobject.threads_init()

class LpBugReporter(gtk.Window):
    
    def __init__(self, project_name):
        """
        Constructor
        """
        assert not project_name == "", "Project name is mandatory"
        
        # Initialize class attributes
        self.__project_name = project_name
        self.__project_name_lower = project_name.lower()
        self.__row_max = 0.0
        self.__is_start_box_visible = True
        self.__has_fetched_bugs = False
        self.__repeat_search = False
        
        # Get traceback
        self.__stringio = StringIO.StringIO()
        traceback.print_exc(file = self.__stringio)
        self.__error_message = self.extract_last_line()
        
        # Initialize GUI
        log.debug("Initializing Launchdpad Bug Reporter")
        self.builder = gtk.Builder()
        self.builder.add_from_file(os.path.join(PROJECT_PATH, "glade", "bugreporter.glade"))
        self.builder.connect_signals(self)
        
        self.__project_link_button = self.builder.get_object("LaunchButton")
        self.__project_link_button_original_text = self.__project_link_button.get_label()
        self.__project_link_button.set_label(self.__project_link_button_original_text % ("Search in", self.__project_name))
        
        self.__treeview_bugs = self.builder.get_object("treeviewBugs")
        self.__liststore_bugs_found = self.builder.get_object("BugsFound")
        
        self.__vbox_start = self.builder.get_object("vboxStart")
        self.__vbox_no_bug = self.builder.get_object("vboxNoBug")
        self.__vbox_no_bug.hide()
        self.__vbox_bugs = self.builder.get_object("vboxBugs")
        self.__vbox_bugs.hide()
        
        self.__label_error_message = self.builder.get_object("labelErrorMessage")
        self.__label_error_message.set_markup("<b><i>%s</i></b>" % self.__error_message)
        self.__label_no_bug_title = self.builder.get_object("labelNoBugTitle")
        self.__label_bug_question = self.builder.get_object("labelBugQuestion")
        self.__label_bug_question_orig = self.__label_bug_question.get_text()
        self.__label_launch_button_help = self.builder.get_object("labelLaunchButtonHelp")
        
        self.__progressbar_bug_fetch = self.builder.get_object("progressbarBugFetch")
        self.__button_stop_start_fetching = self.builder.get_object("buttonStartStopFetching")
        self.__image_stop_start_fetching = self.builder.get_object("imageStartStopFetching")
        
        self.__textview_traceback = self.builder.get_object("textviewTraceback")
        buffer = self.__textview_traceback.get_buffer()
        buffer.set_text(self.__stringio.getvalue())
        
        self.builder.get_object("BugReporterWindow").connect("key-press-event", self.on_key_press)
        self.builder.get_object("BugReporterWindow").connect("key-release-event", self.on_key_release)
        log.debug("Launchpad Bug Reporter initialized !")
        
        self.set_bug_count(0)
        
        gtk.main()
    
    # Property row_max
    def get_row_max(self):
        return self.__row_max
    def set_row_max(self, row_max):
        self.__row_max = float(row_max)
    row_max = property(get_row_max, set_row_max)
    
    def enable_start_stop_fetching_button(self):
        """
        Re-enable the Start / Stop button
        """
        self.__button_stop_start_fetching.set_sensitive(True)
    
    def append_row(self, row):
        """
        Append a new row to the treeviewBugs
        """
        self.__liststore_bugs_found.append(row)
        row_count = len(self.__liststore_bugs_found)
        if row_count > 0:
            self.__has_fetched_bugs = True
            fraction = (row_count / self.row_max)
            gobject.idle_add(self.__progressbar_bug_fetch.set_fraction, fraction)
    
    def switch_view(self):
        """
        Switch view from Start to no bug or bugs
        """
        gobject.idle_add(self.__vbox_start.hide)
        self.__is_start_box_visible = False
        if self.row_max == 0:
            title = self.__label_no_bug_title.get_text() % ("<b>%s</b>" % self.__project_name)
            gobject.idle_add(self.__label_no_bug_title.set_markup, title)
            gobject.idle_add(self.__vbox_no_bug.show)
        else:
            gobject.idle_add(self.__vbox_bugs.show)
            self.set_bug_count(self.row_max)
    
    def return_to_start(self):
        """
        Return to the start scree with error message
        """
        gobject.idle_add(self.__vbox_no_bug.hide)
        gobject.idle_add(self.__vbox_bugs.hide)
        gobject.idle_add(self.__vbox_start.show)
        self.__is_start_box_visible = True
    
    def on_error(self, message):
        """
        Show the import error that happened.
        """
        gobject.idle_add(self.__vbox_start.hide)
        self.__is_start_box_visible = False
        gobject.idle_add(self.__vbox_no_bug.show)
        gobject.idle_add(self.__label_no_bug_title.set_markup, self.__label_no_bug_title.get_text() %
                         ("<b>" + self.__project_name + "</b>"))
    
    def set_bug_count(self, count):
        """
        Update the question label to show the bugs count.
        """
        if count == 1:
            gobject.idle_add(self.__label_bug_question.set_text, "Is this bug similar to yours ?")
        else:
            gobject.idle_add(self.__label_bug_question.set_text, self.__label_bug_question_orig % count)
    
    def extract_last_line(self):
        """
        Extract the latest line of the traceback to use it in bug search
        """
        return self.__stringio.getvalue().split("\n")[-2]
    
    def reset_widgets(self):
        """
        Reset progressbar and Treeview
        """
        self.__liststore_bugs_found.clear()
        self.__progressbar_bug_fetch.set_fraction(0.0)
        self.__has_fetched_bugs = False
    
    def _quit_lp_bug_reporter(self):
        """
        Terminate the application GTK
        """
        gtk.main_quit()
    
    #---------------------------------------------------------------------------
    # GUI callback methods
    #---------------------------------------------------------------------------
    
    def on_LaunchButton_clicked(self, widget=None):
        """
        User clicked on button 'Go to Launchpad and search in bugs'
        """
        if not self.__has_fetched_bugs or self.__repeat_search:
            self.reset_widgets()
            # Initialize Thread and Queue
            self.__thread = None
            self.__thread = LpBrThread(self)
            self.__thread.project = self.__project_name
            self.__thread.cache_dir = os.path.expanduser("~/.launchpadlib/cache")
            self.__thread.start()
        else:
            self.switch_view()
    
    def on_buttonStartStopFetching_clicked(self, widget):
        """
        User click on stop fetching button
        """
        if self.__thread.is_running:
            self.__thread.stop()
            icon = gtk.STOCK_REFRESH
            self.__button_stop_start_fetching.set_sensitive(False)
        else:
            self.reset_widgets()
            self.on_LaunchButton_clicked()
            icon = gtk.STOCK_STOP
        
        gobject.idle_add(self.__image_stop_start_fetching.set_from_stock, icon, gtk.ICON_SIZE_MENU)
    
    def on_QuitButton_clicked(self, widget):
        """
        Quit Button clicked
        """
        self._quit_lp_bug_reporter()
    
    def on_AboutButton_clicked(self, widget):
        """
        Show the About window
        """
        gtk.about_dialog_set_url_hook(self.on_WebSiteLink_clicked)
        
        about = gtk.AboutDialog()
        about.set_transient_for(self)
        about.set_name("Launchdpad Bug Reporter")
        about.set_version(VERSION)
        about.set_icon(gtk.gdk.pixbuf_new_from_file(LOGO_PATH))
        about.set_logo(gtk.gdk.pixbuf_new_from_file(LOGO_PATH))
        about.set_website("https://launchpad.net/lpbugreporter")
        about.set_website_label("Launchdpad Bug Reporter Website")
        about.set_comments("Launchdpad Bug Reporter provide a simple module to report bugs for hosted Launchdpad projects.")
        about.set_authors(["zedtux <zedtux@zedroot.org>"])
        about.set_copyright("Copyright © 2010 zedtux")
        about.set_wrap_license(True)
        about.set_license(("Launchdpad Bug Reporter is free software; " +
                           "you can redistribute it and/or modify it under the terms " +
                           "of the GNU General Public License as published by the Free Software Foundation; " +
                           "either version 3 of the License, or (at your option) any later version.\n" +
                           "Launchdpad Bug Reporter is distributed in the hope that it will be useful, " +
                           "but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  " +
                           "See the GNU General Public License for more details.\n" +
                           "You should have received a copy of the GNU General Public License along with Launchdpad Bug Reporter; " +
                           "if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA"))
        about.run()
        about.destroy()
    
    def on_ReportBugButton_clicked(self, widget):
        """
        When the user clicked on 'Report this bug' button
        """
        self.go_to_url(PROJECT_BUGS_LINK % self.__project_name_lower)
        self._quit_lp_bug_reporter()
    
    def on_WebSiteLink_clicked(self, dialog, link):
        """
        When the user click on the Website link in the About Dialog
        """
        self.go_to_url(link)
    
    def go_to_url(self, link):
        """
        Open a webbrowser instance to the given Url
        """
        webbrowser.open(link)
    
    def on_buttonCancelReport_clicked(self, widget):
        """
        Close the Bug Reporter dialog.
        """
        print "on_buttonCancelReport_clicked"
    
    def on_BugsFoundTreeview_row_activated(self, treeview, path, column):
        """
        When user double-click on a row.
        """
        bug_id = self.__liststore_bugs_found.get_value(self.__liststore_bugs_found.get_iter(path), 0)
        self.go_to_url(PROJECT_BUG_LINK % (self.__project_name_lower, bug_id))
    
    def on_BackButton_clicked(self, widget):
        """
        When is on the bug list screen and click on the 'Back to the error message' button
        """
        self.__project_link_button.set_label(self.__project_link_button_original_text % ("Return to", self.__project_name))
        self.__label_launch_button_help.show()
        self.return_to_start()
    
    def on_key_press(self, widget, event):
        """
        When keyboad key pressed
        """
        if event.state and gtk.gdk.CONTROL_MASK and self.__is_start_box_visible and self.__has_fetched_bugs:
            self.__project_link_button.set_label(self.__project_link_button_original_text % ("Search in", self.__project_name))
            self.__repeat_search = True

    def on_key_release(self, widget, event):
        """
        When keyboad key released
        """
        if event.state and gtk.gdk.CONTROL_MASK and self.__is_start_box_visible and self.__has_fetched_bugs:
            self.__project_link_button.set_label(self.__project_link_button_original_text % ("Return to", self.__project_name))
            self.__repeat_search = False

if __name__ == "__main__":
    enable_debugging()
    try:
        print 2/0
    except:
        LpBugReporter("quiestla")
        try:
            gtk.main()
        except KeyboardInterrupt:
            pass
